#import <Foundation/Foundation.h>
// Declare Packet Type Codes
typedef enum {
    AsReaderP252BPacketResSetProperty = 0x57,        // 'W'
    AsReaderP252BPacketResGetProperty = 0x52,        // 'R'
    AsReaderP252BPacketCommandAck     = 0x41,        // 'A'
    AsReaderP252BPacketEvent          = 0x65,        // 'e'
} AsReaderP252BPacketType;

// Declare Property Type Codes
typedef enum {
    AsReaderP252BPropertyBuzzer                = 0x62,    // 'b'
    AsReaderP252BPropertyContinuousMode        = 0x63,    // 'c'
    AsReaderP252BPropertyFirmwareVersion       = 0x76,    // 'v'
    AsReaderP252BPropertyHardwareVersion       = 0x48,    // 'H'
    AsReaderP252BPropertyPowerGain             = 0x70,    // 'p'
    AsReaderP252BPropertyPowerGainRange        = 0x61,    // 'a'
    AsReaderP252BPropertyRegion                = 0x67,    // 'g'
    AsReaderP252BPropertyOperationTime         = 0x69,    // 'i'
    AsReaderP252BPropertyInventoryTime         = 0x32,    // '2'
    AsReaderP252BPropertyIdleTime              = 0x30,    // '0'
    AsReaderP252BPropertyAutoOffTime           = 0x74,    // 't'
    AsReaderP252BPropertyAccessPassword        = 0x77,    // 'w'
    AsReaderP252BPropertyInventorySession      = 0x73,    // 's'
    AsReaderP252BPropertySessionFlag           = 0x66,    // 'f'
    AsReaderP252BPropertyMask                  = 0x4D,    // 'M'
    AsReaderP252BPropertyUHFVersion            = 0x56,    // 'V'
    AsReaderP252BPropertyBatteryStatus         = 0x65,    // 'e'
    AsReaderP252BPropertySerialNo              = 0x53,    // 'S'
    AsReaderP252BPropertyMinQ                  = 0x4E,    // 'N'
    AsReaderP252BPropertyMaxQ                  = 0x58,    // 'X'
    AsReaderP252BPropertyEpcMask               = 0x64,    // 'd'
    AsReaderP252BPropertyEpcMaskData           = 0x44,    // 'D'
    AsReaderP252BPropertyLBT                   = 0x4C,    // 'L'
    AsReaderP252BPropertyLBTMask               = 0x39,    // '9'
    AsReaderP252BPropertyQValue                = 0x51,    // 'Q'
    AsReaderP252BPropertyLinkProfile           = 0x34,    // '4'
    AsReaderP252BPropertySleepTime             = 0x79,    // 'y'
    AsReaderP252BPropertyBarcodeTimeOut        = 0x40,    // '@'
    AsReaderP252BPropertyBaudRateList          = 0x49,    // 'I'
    AsReaderP252BPropertyBaudRate              = 0x6D,    // 'm'
} AsReaderP252BPropertyType;

// Declare Sub Command Type Codes
typedef enum {
    AsReaderP252BCommandInventory       = 0x66,    		// 'f'
    AsReaderP252BCommandReadMemory      = 0x72,    		// 'r'
    AsReaderP252BCommandWriteMemory     = 0x77,         // 'w'
    AsReaderP252BCommandKill            = 0x6B,    		// 'k'
    AsReaderP252BCommandLock            = 0x6C,    		// 'l'
    AsReaderP252BCommandUnlock          = 0x6D,   	    // 'm'
    AsReaderP252BCommandPermaLock       = 0x70,         // 'p'
    AsReaderP252BCommandStop            = 0x73,    		// 's'
    AsReaderP252BCommandDefaultParam    = 0x61,         // 'a'
    AsReaderP252BCommandSaveParam       = 0x53,         // 'S'
    AsReaderP252BCommandBuzzerStart     = 0x75,         // 'u'
    AsReaderP252BCommandDecodeStart     = 0x64,         // 'd'
} AsReaderP252BCommandType;

// Declare Event Type Codes
typedef enum {
    AsReaderP252BEventTagValue      = 0x54,           // 'T'
    AsReaderP252BEventAccessResult  = 0x41,           // 'A'
    AsReaderP252BEventRemoteKey     = 0x42,           // 'B'
    AsReaderP252BEventReaderValue   = 0x52,           // 'R'
    AsReaderP252BEventHTTagValue    = 0x48,           // 'U'
    AsReaderP252BEventBarcodeValue  = 0x43,           // 'C'
} AsReaderP252BEventType;
 
typedef enum {
    AsReaderP252BRFIDScanMode,
    AsReaderP252BBarcodeScanMode,
} AsReaderP252BScanMode;
@interface AsReaderP252BPacket : NSObject

@property (nonatomic) NSInteger errorCode;
@property (strong, nonatomic) NSData *data;

- (id)initWithPacketType:(AsReaderP252BPacketType)type subType:(uint8_t)subType;
- (AsReaderP252BPacketType)type;
- (uint8_t)subType;
- (void)signal;
- (void)wait;
- (BOOL)equalPacket:(AsReaderP252BPacketType)type subType:(uint8_t)subType;
- (BOOL)checkTimeoutInterval:(int)interval withTimeout:(int)maxTimeout;

@end

@interface AsReaderP252BPacketManager : NSObject

- (void)addPacket:(AsReaderP252BPacket *)packet;
- (AsReaderP252BPacket *)waitPacket:(AsReaderP252BPacketType)type subType:(uint8_t)subType;
- (AsReaderP252BPacket *)findPacket:(AsReaderP252BPacketType)type subType:(uint8_t)subType;
- (AsReaderP252BPacket *)firstPacket;
- (void)removePacket:(AsReaderP252BPacket *)packet;
- (BOOL)isWaitPacket;

@end
